#!/bin/sh
# centro_control.sh - Programa para realizar diversas tareas de administracin
# del servidor de un centro Guadalinfo: identificacin del centro, control
# de las MACs de los clientes, copias de seguridad, habilitacin de acceso
# externo, habilitacin de acceso a los clientes,...

# Estamos en X?
if [ ! $DISPLAY ]; then
  : ${DIALOG=dialog}
else
  : ${DIALOG=Xdialog}
fi

menu_principal()
{
  tempfile=`tempfile 2>/dev/null` || tempfile=/tmp/test$$
  trap "rm -f $tempfile" 0 1 2 5 15
  
  $DIALOG --clear --cancel-label "Salir" \
  	  --title "Centro de control" \
          --menu "Seleccione una de las siguientes opciones \n
para configurar diferentes aspectos del servidor de Guadalinfo \n\n"	0 0 8 \
          "MACs" "Direcciones MACs de los clientes" \
	  "Acceso" "Control de acceso remoto a los clientes" \
          "ID"  "Identificacin del centro Guadalinfo" \
	  "Quotas" "Gestin de quotas de usuarios" \
	  "Backup" "Copias de seguridad de servidor y clientes" \
	  "Estadsticas" "Envo de los ficheros de estadsticas" \
	  "Entrada" "Control de entrada externa al servidor" 2> $tempfile
  
  retval=$?
  
  choice=`cat $tempfile`
  
}

menu_id()
{
  CONF_FILE=/etc/guadalinfo_id.conf
# Por si no existe, lo creamos
  touch $CONF_FILE
# Cargamos el contenido del fichero de identificacin
  . $CONF_FILE

# Se distingue entre dialog/Xdialog
  if [ $DIALOG = "dialog" ]; then
    exec 3>&1
    value="`$DIALOG --ok-label "Aceptar" \
          --backtitle "Parmetros de identificacin del centro" \
          --form "Edite los parmetros de identificacin del centro:" \
12 70 0 \
        "Nombre de la localidad:" 	 1 1 "$NOMBRE_LOCALIDAD" 1 20 30 0 \
        "Identificador de la localidad:" 2 1 "$ID_LOCALIDAD"  	 2 27  5 0 \
        "Nombre de la provincia:"	 3 1 "$NOMBRE_PROVINCIA" 3 25 10 0 \
        "Identificador de la provincia:" 4 1 "$ID_PROVINCIA" 	 4 32  5 0 \
    2>&1 1>&3`"
    returncode=$?
    exec 3>&-

    if [ $returncode = "0" ]; then # Se ha pulsado aceptar
      #show=`echo "$value" |sed -e 's/^/       /'`
      echo -e "$value" > "/tmp/gl_id.tmp"
      NOMBRE_LOCALIDAD=`sed -n '1p' /tmp/gl_id.tmp`
      ID_LOCALIDAD=`sed -n '2p' /tmp/gl_id.tmp`
      NOMBRE_PROVINCIA=`sed -n '3p' /tmp/gl_id.tmp`
      ID_PROVINCIA=`sed -n '4p' /tmp/gl_id.tmp`
      escribir="si"
    fi
  else # DIALOG = Xdialog
# 1a ventana: nombre de la localidad
    $DIALOG --title "Identificacion del centro" \
            --inputbox "Introduzca el nombre de la localidad del centro:" \
	     0 0 "$NOMBRE_LOCALIDAD" 2> /tmp/inputbox.tmp.$$
    
    retval=$?
    
    input=`cat /tmp/inputbox.tmp.$$`
    rm -f /tmp/inputbox.tmp.$$
    
    case $retval in
      0)
        NOMBRE_LOCALIDAD="$input";;
      *)
        return 1;;
    esac
# 2a ventana: identificacin de la localidad
    $DIALOG --title "Identificacion del centro" \
            --inputbox "Introduzca el identificador de la localidad del centro:" \
            0 0 "$ID_LOCALIDAD" 2> /tmp/inputbox.tmp.$$
    
    retval=$?
    
    input=`cat /tmp/inputbox.tmp.$$`
    rm -f /tmp/inputbox.tmp.$$
    
    case $retval in
      0)
        ID_LOCALIDAD="$input";;
      *)
        return 1;;
    esac
# 3a ventana: nombre de la provincia
    $DIALOG --title "Identificacion del centro" \
            --inputbox "Introduzca el nombre de la provincia:" \
            0 0 "$NOMBRE_PROVINCIA" 2> /tmp/inputbox.tmp.$$
    
    retval=$?
    
    input=`cat /tmp/inputbox.tmp.$$`
    rm -f /tmp/inputbox.tmp.$$
    
    case $retval in
      0)
        NOMBRE_PROVINCIA="$input";;
      *)
        return 1;;
    esac
# 4a ventana: identificador de la provincia
    $DIALOG --title "Identificacion del centro" \
            --inputbox "Introduzca el identificador de la provincia:" \
            0 0 "$ID_PROVINCIA" 2> /tmp/inputbox.tmp.$$
    
    retval=$?
    
    input=`cat /tmp/inputbox.tmp.$$`
    rm -f /tmp/inputbox.tmp.$$
    
    case $retval in
      0)
        ID_PROVINCIA="$input";;
      *)
        return 1;;
    esac

# Habilitamos la escritura en el fichero de configuracin
    escribir="si"
  fi

# Se cambia el mensaje de bienvenida del GDM y se enva a los clientes
  gdmplantilla.py "$NOMBRE_LOCALIDAD" > /usr/share/gdm/themes/guadalinex/guadalinex.xml

  i=1
  while [ $i -le 20 ]
  do
    scp /usr/share/gdm/themes/guadalinex/guadalinex.xml cliente-$i:/usr/share/gdm/themes/guadalinex/guadalinex.xml&
    pid[$i]=`pidof scp /usr/share/gdm/themes/guadalinex/guadalinex.xml cliente-$i:/usr/share/gdm/themes/guadalinex/guadalinex.xml`
    let i=$i+1
  done

# Esperamos a que se produzcan las copias remotas
  sleep 2

  i=1
  while [ $i -le 20 ]
  do
    kill -9 ${pid[$i]}
    let i=$i+1
  done
 
  if [ "$escribir" = "si" ]; then  
    # Volcamos los datos en el fichero de configuracin
    echo "NOMBRE_LOCALIDAD=\"$NOMBRE_LOCALIDAD\"" > $CONF_FILE
    echo "ID_LOCALIDAD=\"$ID_LOCALIDAD\"" >> $CONF_FILE
    echo "NOMBRE_PROVINCIA=\"$NOMBRE_PROVINCIA\"" >> $CONF_FILE
    echo "ID_PROVINCIA=\"$ID_PROVINCIA\"" >> $CONF_FILE
  fi

# Mostramos un resumen con los valores introducidos
  $DIALOG --left --title "Identificacion del centro" \
          --msgbox " \
Nombre de la localidad:	        $NOMBRE_LOCALIDAD \n \
Identificador de la localidad:  $ID_LOCALIDAD \n \
Nombre de la provincia:         $NOMBRE_PROVINCIA \n \
Identificador de la provincia:  $ID_PROVINCIA" 0 0 10000
  return 0
}

muestra_macs()
{
    if [ $DIALOG = "dialog" ]; then
      exec 3>&1
      value="`$DIALOG --ok-label "Aceptar" --no-cancel \
            --backtitle "Configuracin de las MACs de los clientes" \
            --form "Modifique a continuacin las direcciones MACs de los \
  clientes que desee:" 0 50 23 \
          "  MAC cliente-1:" 1 1 "${mac[1]}" 1 20 17 17 \
          "  MAC cliente-2:" 2 1 "${mac[2]}" 2 20 17 17 \
          "  MAC cliente-3:" 3 1 "${mac[3]}" 3 20 17 17 \
          "  MAC cliente-4:" 4 1 "${mac[4]}" 4 20 17 17 \
          "  MAC cliente-5:" 5 1 "${mac[5]}" 5 20 17 17 \
          "  MAC cliente-6:" 6 1 "${mac[6]}" 6 20 17 17 \
          "  MAC cliente-7:" 7 1 "${mac[7]}" 7 20 17 17 \
          "  MAC cliente-8:" 8 1 "${mac[8]}" 8 20 17 17 \
          "  MAC cliente-9:" 9 1 "${mac[9]}" 9 20 17 17 \
          "  MAC cliente-10:" 10 1 "${mac[10]}" 10 20 17 17 \
          "  MAC cliente-11:" 11 1 "${mac[11]}" 11 20 17 17 \
          "  MAC cliente-12:" 12 1 "${mac[12]}" 12 20 17 17 \
          "  MAC cliente-13:" 13 1 "${mac[13]}" 13 20 17 17 \
          "  MAC cliente-14:" 14 1 "${mac[14]}" 14 20 17 17 \
          "  MAC cliente-15:" 15 1 "${mac[15]}" 15 20 17 17 \
          "  MAC cliente-16:" 16 1 "${mac[16]}" 16 20 17 17 \
          "  MAC cliente-17:" 17 1 "${mac[17]}" 17 20 17 17 \
          "  MAC cliente-18:" 18 1 "${mac[18]}" 18 20 17 17 \
          "  MAC cliente-19:" 19 1 "${mac[19]}" 19 20 17 17 \
          "  MAC cliente-20:" 20 1 "${mac[20]}" 20 20 17 17 \
          "  MAC impresora:"  22 1 "${mac[0]}" 22 20 17 17 \
      2>&1 1>&3`"
      returncode=$?
      exec 3>&-
  
  # Guardamos los valores introducidos en un fichero por filas
      echo -e "$value" > "/tmp/gl_id.tmp"
  # Leemos el fichero por filas y almacenamos sus valores en variables
      mac[1]=`sed -n '1p' /tmp/gl_id.tmp`
      mac[2]=`sed -n '2p' /tmp/gl_id.tmp`
      mac[3]=`sed -n '3p' /tmp/gl_id.tmp`
      mac[4]=`sed -n '4p' /tmp/gl_id.tmp`
      mac[5]=`sed -n '5p' /tmp/gl_id.tmp`
      mac[6]=`sed -n '6p' /tmp/gl_id.tmp`
      mac[7]=`sed -n '7p' /tmp/gl_id.tmp`
      mac[8]=`sed -n '8p' /tmp/gl_id.tmp`
      mac[9]=`sed -n '9p' /tmp/gl_id.tmp`
      mac[10]=`sed -n '10p' /tmp/gl_id.tmp`
      mac[11]=`sed -n '11p' /tmp/gl_id.tmp`
      mac[12]=`sed -n '12p' /tmp/gl_id.tmp`
      mac[13]=`sed -n '13p' /tmp/gl_id.tmp`
      mac[14]=`sed -n '14p' /tmp/gl_id.tmp`
      mac[15]=`sed -n '15p' /tmp/gl_id.tmp`
      mac[16]=`sed -n '16p' /tmp/gl_id.tmp`
      mac[17]=`sed -n '17p' /tmp/gl_id.tmp`
      mac[18]=`sed -n '18p' /tmp/gl_id.tmp`
      mac[19]=`sed -n '19p' /tmp/gl_id.tmp`
      mac[20]=`sed -n '20p' /tmp/gl_id.tmp`
      mac[0]=`sed -n '21p' /tmp/gl_id.tmp`

    else # DIALOG = Xdialog
# Preparamos el listado de clientes y MACs que se va a mostrar
      i=1 
      cadena=""
      while [ $i -le $MAX_CLIENTES ]
      do
        if [ "${mac[$i]}" = "" ]; then
	  mac[$i]="00:00:00:00:00:00"
	fi
# Eliminamos cualquier posible carcter "espacio" que pueda haber en las variables MACS
	mac[$i]=${mac[$i]//" "/""}
        cadena="$cadena cliente-$i \"${mac[$i]}\""
	let i=$i+1
      done
# Aadimos la impresora
      if [ "${mac[0]}" = "" ]; then
	 mac[0]="00:00:00:00:00:00"
      fi
      mac[0]=${mac[0]//" "/""}

      cadena="$cadena Impresora \"${mac[0]}\""
      
      $DIALOG --title "Configuracion de las direcciones MACs" \
      	      --cancel-label "Salir" --ok-label "Seleccionar" \
              --menu "Seleccione a continuacin alguno de los clientes \n
      para establecer su direccin MAC:" 0 0 0 \
              $cadena 2> /tmp/menu.tmp.$$
      
      retval=$?
      
      choice=`cat /tmp/menu.tmp.$$`
      rm -f /tmp/menu.tmp.$$
      
      case $retval in
        0) # Se pide la MAC seleccionada
	  return 0
          ;;
        *)
          return 1
	  ;;
      esac
    fi
    return 0
}

menu_macs()
{
# Se leen todas las MACs de los clientes
  MAX_CLIENTES=20
  RUTA=/etc/dhcp3
  
  i=1;
  while [ $i -le $MAX_CLIENTES ]
  do
    # Leemos la segunda lnea de cada fichero
    linea=`sed -n '2p' $RUTA/mac_cliente$i.conf`
    # Le quitamos los espacios al principio
    temp=${linea#"   "}
    # Le quitamos "hardware ethernet"
    patron="hardware ethernet "
    temp=${temp#$patron}
    # Le quitamos el ; y lo almacenamos en su variable correspondiente
    patron=";"
    mac[$i]=${temp%$patron}
    let i=$i+1
  done
# La impresora
    # Leemos la segunda lnea del fichero
    linea=`sed -n '2p' $RUTA/mac_impresora.conf`
    # Le quitamos los espacios al principio
    temp=${linea#"   "}
    # Le quitamos "hardware ethernet"
    patron="hardware ethernet "
    temp=${temp#$patron}
    # Le quitamos el ; y lo almacenamos en su variable correspondiente
    patron=";"
    mac[0]=${temp%$patron}
    let i=$i+1
    

# Se muestran en el formulario (distinguiendo dialog/Xdialog)
    muestra_macs
    return_value=$?
    cliente=$choice
    num_cliente=${choice//"cliente-"/""}

# Lo que viene a continuacin slo se hace si estamos con Xdialog
    if [ "$DIALOG" = "Xdialog" ]; then
      if [ "$return_value" = "0" ]; then
        
# Si   se ha seleccionado alguna MAC, se pide al usuario su valor
        $DIALOG --title "Introduccion de MACs" \
        	      --ok-label "Cambiar" \
                --inputbox "Introduzca la direccin MAC del \n
                cliente $num_cliente:" \
                0 0 ${mac[$num_cliente]} 2> /tmp/inputbox.tmp.$$

        retval=$?

        input="`cat /tmp/inputbox.tmp.$$`"
        rm -f /tmp/inputbox.tmp.$$

        case $retval in
          0)
            mac[$num_cliente]="$input"
            mac_modificada=1
            ;;
          *)
            return 1;;
        esac

      else # Si no se ha seleccionado ninguna MAC, no se hace nada
        continue
      fi

    fi

# Se escriben las variables en los ficheros
    i=1
    while [ $i -le $MAX_CLIENTES ]
    do
      echo "host cliente-$i {" > $RUTA/mac_cliente$i.conf
      echo "   hardware ethernet ${mac[$i]};" >> $RUTA/mac_cliente$i.conf
      echo "   fixed-address cliente-$i.guadalinfo.guadalinex.org;" >> $RUTA/mac_cliente$i.conf 
      echo "   option host-name \"cliente-$i\";" >> $RUTA/mac_cliente$i.conf 
      echo "}" >> $RUTA/mac_cliente$i.conf
      let i=$i+1
    done
# La impresora
      echo "host laser-printer {" > $RUTA/mac_impresora.conf
      echo "   hardware ethernet ${mac[0]};" >> $RUTA/mac_impresora.conf
      echo "   fixed-address laser-printer.guadalinfo.guadalinex.org;" >> $RUTA/mac_impresora.conf 
      echo "   option host-name \"laser-printer\";" >> $RUTA/mac_impresora.conf 
      echo "}" >> $RUTA/mac_impresora.conf

# Una vez se han cambiado los ficheros, se reinicia el servidor de DHCP
    /etc/init.d/dhcp3-server restart

# Si se cambi alguna MAC, se escribe en el fichero correspondiente y se vuelve al men
# de las MACs
    if [ "$mac_modificada" = "1" ]; then
      menu_macs
    fi

    return 0
}

menu_entrada()
{
# Men correspondiente a la gestin del acceso externo al servidor via SSH
  habilitar_acceso_externo.sh
  return 0
}

menu_acceso()
{
# Men correspondiente a la gestin del acceso remoto a los clientes

# Averiguamos el estado de los clientes
# $cadena almacenar lo que se va a mostrar en el Xdialog despus
  cadena=""
  i=1
( while [ $i -le 20 ]
  do
    echo "XXX"
    echo "Buscando cliente-$i..."
    echo "XXX"
    fping -t200 -c 1 -q cliente-$i > /dev/null 2> /dev/null
    if [ $? != "0" ] ; then # El cliente no est presente
      cliente[$i]="No-presente"
    else # El cliente est presente: avegiguamos su estado
# Para reducir tiempos de espera, se accede a los equipos a travs de la IP
      ip=`host cliente-$i | awk '{print $4}'`
#      ssh cliente-$i ls > /dev/null 2> /dev/null &
      ssh $ip ls > /dev/null 2> /dev/null &
      sleep 1
#      pid="`pidof ssh cliente-$i`"
      pid="`pidof ssh $ip`"
      kill -9 $pid
#      ssh cliente-$i ls > /dev/null 2> /dev/null &
      ssh $ip ls > /dev/null 2> /dev/null &
      sleep 1
#      pid="`pidof ssh cliente-$i`"
      pid="`pidof ssh $ip`"
# Si el PID es "", el cliente tiene habilitado el acceso remoto
      if [ "$pid" = "" ]; then
        cliente[$i]="Habilitado"
      else
        kill -9 $pid
        cliente[$i]="Deshabilitado"
      fi
    fi
    cadena="${cadena}cliente-$i ${cliente[$i]} "
# Escribimos la variable $cadena en un fichero para poder acceder desde fuera de 
# este bloque
    echo $cadena > /tmp/acceso.tmp

    let i=$i+1
  done ) | $DIALOG --title "Buscando clientes..." --infobox "" 0 0 0

# Se recupera el contenido de la variable $cadena del fichero
  cadena=`cat /tmp/acceso.tmp`

# Mostramos el estado de todos los clientes

$DIALOG --title "Estado de los clientes" \
        --ok-label "Seleccionar" --cancel-label "Salir" \
        --menu "A continuacin, se muestra el estado \n\
de todos los clientes del centro.\n\n \
Seleccione alguno para cambiar su estado: " 0 0 0 \
     $cadena         2> /tmp/menu.tmp.$$

  retval=$?
  
  choice=`cat /tmp/menu.tmp.$$`
  rm -f /tmp/menu.tmp.$$
  
  case $retval in
    0) # Se ha elegido un cliente
      xterm -e habilitar_acceso_remoto.sh $choice
      menu_acceso
      ;;
    *)
      return 1
  esac 
  
    return 0
}

menu_backup()
{
# Men correspondiente a la gestin de las copias de seguridad en servidor y clientes
  $DIALOG --title "Copias de seguridad" \
          --menu "Seleccione a continuacin si desea hacer una copia de \n\
seguridad de los clientes o del propio servidor:" 0 0 4 \
        "clientes" "Hacer copia de seguridad de los clientes" \
          "servidor"  "Hacer copia de seguridad del servidor" 2> /tmp/backup.tmp.$$

  retval=$?

  choice=`cat /tmp/backup.tmp.$$`
  rm -f /tmp/backup.tmp.$$

  case $retval in
    0) # Se ha seleccionado alguna de las dos entradas
      if [ "$choice" = "clientes" ]; then
        hacer_backup_clientes.sh
      else
        duplicar_sistema.sh
      fi
      menu_backup
      ;;
    *)
      return 1
      ;;
  esac
    return 0
  return 0
}

menu_estadisticas()
{
# Men correspondiente al envo de los ficheros de logs de entradas/salidas y listado
# de usuarios de LDAP
  . /etc/email_estadisticas.conf

  $DIALOG --title "Envio de estadisticas" \
          --ok-label "Seleccionar" --cancel-label "Salir" \
          --menu "Seleccione si desea realizar un nuevo envo de los ficheros \n\
de estadsticas o cambiar la direccin de envo:" 0 0 4 \
        "enviar" "Enva ficheros de estadsticas" \
        "cambiar"  "$TO_ADDR" 2> /tmp/envio.tmp.$$ 

  retval=$?

  choice=`cat /tmp/envio.tmp.$$`
  rm -f /tmp/envio.tmp.$$

  case $retval in
    0) # Se ha seleccionado alguna de las dos entradas
      if [ "$choice" = "enviar" ]; then
        /etc/cron.monthly/logs # Realiza un nuevo envo
        $DIALOG  --title "Envio de estadisticas" \
                 --msgbox "Se han enviado las estadsticas correctamente \n
a la direccin:\n\n
$TO_ADDR" 0 0 10000
	
      else
        $DIALOG --title "Cambio de direccion de envio" \
        --inputbox "Introduzca la direccin de correo donde \n\
desea que se enven las estadsticas a partir de ahora:" \
             0 0 $TO_ADDR 2> /tmp/envio.tmp.$$ 

        retval=$?
    
        input=`cat /tmp/envio.tmp.$$`
        rm -f /tmp/envio.tmp.$$
    
        case $retval in
          0)
            TO_ADDR="$input"
    	    echo "TO_ADDR=\"$TO_ADDR\"" > /etc/email_estadisticas.conf
	    $DIALOG --title "Cambio de direccion de envio" \
	    --msgbox "La direccin de envo de estadsticas se ha cambiado a:\n\
$TO_ADDR" 0 0 0 
            menu_estadisticas
     	    ;;
          *)
            return 1;;
        esac
 
      fi
      ;;
    *)
      return 1
      ;;
  esac

  return 0
}

menu_quotas()
{
# Men correspondiente a la gestin de quotas de usuarios en el servidor
# Se pregunta entre "quota_usuario" o "quota_a_todos"
  $DIALOG --title "Gestion de quotas" \
  	  --ok-label "Seleccionar" --cancel-label "Salir" \
          --menu "Seleccione a continuacin si desea cambiar la quota de disco \n\
  de un nico usuario o de todos los usuarios del sistema:" 0 0 4 \
  	"usuario" "Cambiar la quota de un usuario" \
          "todos"  "Cambiar la quota de todos los usuarios" 2> /tmp/quota.tmp.$$
  
  retval=$?
  
  choice=`cat /tmp/quota.tmp.$$`
  rm -f /tmp/quota.tmp.$$
  
  case $retval in
    0) # Se ha seleccionado alguna de las dos entradas
      if [ "$choice" = "usuario" ]; then
	quota_usuario.sh
      else
        quota_a_todos.sh
      fi
      menu_quotas
      ;;
    *)
      return 1
      ;;
  esac
    return 0
}

fin=0

while [ "$fin" != "1" ]
do
  menu_principal

  if [ "$retval" = "1" -o "$retval" = "255" ]; then
    fin=1
  else
    case "$choice" in
      "ID")
        menu_id
	;;
      "MACs")
        menu_macs
	;;
      "Entrada")
        menu_entrada
	;;
      "Acceso")
        menu_acceso
	;;
      "Backup")
        menu_backup
	;;
      "Estadsticas")
        menu_estadisticas
	;;
      "Quotas")
        menu_quotas
	;;
      *)
        fin=1
	;;	
    esac	
  fi
  
done

