#!/usr/bin/env python
import string
import re
import crypt
import datetime
from datetime import date

months = {'jan': '01', 'feb': '02', 'mar': '03', 'apr': '04', 'may': '05', 'jun': '06', 'jul': '07', 'aug': '08', 'sep': '09', 'oct': '10', 'nov': '11', 'dec': '12', 'ene': '01', 'abr': '04', 'ago': '08', 'dic': '12'}

# Get 'localidad' and 'provincia' from config file
def read_conf(file):
	f = open(file, "r")
	f_content = f.read()
# Cambiado para que funcione con ids no numricas
#	result = re.findall('ID_LOCALIDAD="(\d+)"', f_content)
	result = re.findall('ID_LOCALIDAD="(\S+)"', f_content)
	localidad = result[0]
#	result = re.findall('ID_PROVINCIA="(\d+)"', f_content)
	result = re.findall('ID_PROVINCIA="(\S+)"', f_content)
	provincia = result[0]
	return (localidad, provincia)

# Read lines from log file and return logins and logout as event objects
class AuthParser:
	def __init__(self, file):
		self.source = open(file, "r")
		passwd = open("/etc/passwd", "r")
		self.passwd_content = passwd.read()
		passwd.close()
	def in_passwd(self, user):
		return re.search(user, self.passwd_content)
	def parse_line(self):
		line = self.source.readline()
		while line:
			if re.search('(opened)', line):
				# Apr 13 07:53:01 seraphim cron(pam_unix)[9909]: session opened for user mail by (uid=0)
				pattern = '(\S+) +(\d+) +(\d+):(\d+):(\d+) +(\S+) +(\w+)\((\S+)\)\[(\d+)\]: session opened for user (\S+)'
				type = "opened"
			elif re.search('(closed)', line):
				# Apr 13 07:53:01 seraphim cron(pam_unix)[9909]: session closed for user mail
				pattern = '(\S+) +(\d+) +(\d+):(\d+):(\d+) +(\S+) +(\w+)\((\S+)\)\[(\d+)\]: session closed for user (\S+)'
				type = "closed"
			else:
				line = self.source.readline()
				continue
			# result[0][0]: month, result[0][1]: day, result[0][2]: hours, result[0][3]: minutes, result[0][4]: seconds,
			# result[0][5]: source, result[0][6]: n/a, result[0][7]: n/a, result[0][8]: id, result[0][9]: user
			result = re.findall(pattern, line);
			if result and (result[0][9] == "usuario" or result[0][9] == "root" or not self.in_passwd(result[0][9])):
				user = crypt.crypt(result[0][9], "dc")
				data = (result[0][0], result[0][1], result[0][2], result[0][3], result[0][4], result[0][5], result[0][8], user)
				item = Event(type, data)
				return item
			line = self.source.readline()

# Holds one login or logout
class Event:
	def __init__(self, type = "any", data = ['?', '?', '?', '?', '?', '?', '?', '?']):
		self.type = type
		self.month = data[0]
		self.day = data[1]
		self.hours = data[2]
		self.minutes = data[3]
		self.seconds = data[4]
		self.source = data[5]
		self.id = data[6]
		self.user = data[7]

# Holds one Sessions object per source (ip or hostname)
class Report:
	def __init__(self):
		self.source_list = {}
	def in_report(self, source_id):
		return self.source_list.has_key(source_id)
	def add(self, event):
		if not self.in_report(event.source):
			self.source_list[event.source] = Sessions()
		self.source_list[event.source].add(event)
	def display(self, sep):
		result = ""
		for source in self.source_list:
			sessions = self.source_list[source]
			result = result + sessions.display(source, sep)
		return result

# Holds a list of login events, a list of logout events and a list to prevent multiple event ids
class Sessions:
	def __init__(self):
		self.opened = {}
		self.closed = {}
		self.id_table = {}
	def add(self, event):
		if event.type == "opened":
			new_id = int(event.id)
			# Finds new id (previously unallocated) if needed
			while (self.opened.has_key(str(new_id))):
				new_id = new_id + 100000
			# Stores event in list
			self.opened[str(new_id)] = event
			# If new id, stores it in id_table
			if event.id != str(new_id):
				self.id_table[event.id] = str(new_id)
		elif event.type == "closed":
			new_id = event.id
			if self.id_table.has_key(event.id):
				new_id = self.id_table[event.id]
			self.closed[new_id] = event
		else:
			return
		last_event.month = event.month
		last_event.day = event.day
	def display(self, source, sep):
		result = ""
		for event_id in self.opened:
			open_event = self.opened[event_id]
			result = result + string.join((provincia, localidad, source, open_event.user, open_event.month, open_event.day, string.join((open_event.hours, open_event.minutes, open_event.seconds), ":")), sep)
			if self.closed.has_key(event_id):
				close_event = self.closed[event_id]
				result = result + sep + string.join((close_event.month, close_event.day, string.join((close_event.hours, close_event.minutes, close_event.seconds), ":")), sep) + sep + now.strftime("%m")
			else:
				result = result + sep + "?" + sep + "?" + sep + "?" + sep + now.strftime("%m")
			result = result + "\n"
		return result

now = date.today()

last_event = Event()

(localidad, provincia) = read_conf("/etc/guadalinfo_id.conf")

parser = AuthParser("/var/log/auth.log")

guadalinfo = Report()

event = parser.parse_line()
while (event):
	guadalinfo.add(event)
	event = parser.parse_line()

date = datetime.date.today()
split_date = string.split(str(date), "-")

try:
	f_dest = open("/tmp/" + provincia + "_" + localidad + "_" + split_date[0] + months[last_event.month.lower()] + last_event.day + "_uso", "w")
except :
	f_dest = open("/tmp/" + provincia + "_" + localidad + "_" + "0000" + "00" + "0" + "_uso", "w")
	
f_dest.write(guadalinfo.display(";"))
f_dest.close()

